<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.13 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Formatter.php,v 1.13 2014-01-14 20:38:06 mike Exp $
 */

/**
 * Gadget_Formatter
 *
 * Provides formatting routines.
 * 
 * @package     Gadget
 * @since       1.0
 */
class Gadget_Formatter
{
    // {{{ Text Preparation
    /**
     * Prepare Smilies
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $txt
     */
    function prepareSmilies($txt)
    {
        return Gadget_Formatter::replaceSmilies(Gadget_Formatter::prepareFull($txt));
    }

    /**
     * Prepare Mail
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $text
     * @param   string  $charset
     */
    function prepareMail($text, $charset = 'us-ascii')
    {
        return Gadget_Formatter::prepareFull(iconv($charset, GADGET_CHARSET."//TRANSLIT", $text));
    }
    
    /**
     * Prepare Full
     * 
     * Prepare text to be displayed as XHTML.
     * Applies:
     *      o Gadget_Formatter::tidy()
     *      o Gadget_Formatter::replaceLinks()
     *      o Gadget_Formatter::replaceTags()
     *      o Gadget_Formatter::prepareSimple()
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $txt
     */
    function prepareFull($txt)
    {
        $txt = Gadget_Formatter::replaceLinks(
               Gadget_Formatter::replaceTags(
               /*Gadget_Formatter::prepareSimple*/($txt)));
        return Gadget_Formatter::tidy('<p>'. $txt .'</p>');
    }
    
    /**
     * Tidy
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $txt
     */
    function tidy($txt)
    {
        if (!PEAR::loadExtension('tidy')) {
            return $txt;
        }
        // fix ugly duoble breaks
        return preg_replace('/\<br \/\>\s*\<br \/\>/', '',
            tidy_repair_string($txt, Gadget::getConf('Tidy')));
    }
    
    /**
     * Prepare Simple
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $txt
     * @param   bool    $nl2br
     */
    function prepareSimple($txt)
    {
        return nl2br(htmlSpecialChars(trim($txt), 
				ENT_COMPAT|ENT_XHTML, GADGET_CHARSET));
    }
    // }}}
    
    // {{{ Special Replacements
    /**
     * Replace Smilies
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $txt
     */
    function replaceSmilies($txt)
    {
        static $path, $simg;
        
        if (!isset($path, $simg)) {
            $path = Gadget::getConf('Gadget_Formatter', 'smily_path');
            $simg = Gadget::getConf('Gadget_Formatter', 'smily_names');
        }
        
        return preg_replace('/(?<!=")(\:('. $simg .'))/', 
            '<img alt="\\1" src="'. $path .'/\\2.gif" />', $txt);
    }
    
    /**
     * Replace Tags
     * 
     * Replaces [bla] tags with their HTML equivalent, much faster 
     * and way more reliable than BBCodeParser.
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $txt
     */
    function replaceTags($txt)
    {
        static $trules;
        
        if (!isset($trules)) {
            if ($file = Gadget::getConf('Gadget_Formatter', 'tagrules')) {
                require_once 'Gadget/Formatter/Tagrules.php';
                    
                $parser = &new Gadget_Formatter_Tagrules;
                $trules = $parser->parseRules($file);
                unset($parser);
            }
        }
        
        return preg_replace(array_keys($trules), $trules, $txt);
    }
    
    /**
     * Replace Links
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $txt
     */
    function replaceLinks($txt)
    {
        static
        #$pcre = '{((?<!src="|href="|url="|action="|src=|href=|url=|action=)http:\/\/|https:\/\/|ftp:\/\/|news:\/\/|((?<!http:\/\/|https:\/\/)www\.))([\w\-.]+[^,.;:?!/<\s])([/\w.#&=%+@~*_\-,;:]*)([/\w?.#&=%+@~*_\-,;:]*?)(?=[,.;:?!<]*([^/\w.#?&=%+@~*_\-,;:]|$))}ie';
        $pcre = '/((?<!src="|href="|url="|action="|src=|href=|url=|action=)http:\/\/|https:\/\/|ftp:\/\/|news:\/\/|((?<!http:\/\/|https:\/\/)www\.))([\w\-.]+[^,.;:?!\/\s\]!"\)<>])([\w#&=%+@~*_\-,;:]*)([\/\w?.#&=%+@~*_\-,;:]*?)(?=[,.;:?!\]<>!")]*([^\/\w.#?&=%+@~*_\-,;:]|&gt;|$))/i';
        return preg_replace_callback($pcre, array('Gadget_Formatter', 'buildLink'), $txt);
    }
    
    /**
     * Build Link
     * 
     * @static
     * @access  protected
     * @return  string
     * @param   array   $params
     */
    function buildLink($params)
    {
        $params = array_slice($params, 1, -1);

        // catch www.
        if (!strcasecmp($schema = array_shift($params), 'www.')) {
            $schema = 'http://';
        }
        
        $link   = implode('', $params);
        $title  = Gadget_Formatter::shortenText($link);
        $link   = $schema . $link;
        $class  = 'internal';
        $target = 'self';
        
        // external link?
        if (strcasecmp($params[1], Gadget::getConf('Gadget', 'wwwdomain'))) {
            $class  = 'external';
            if ($exttgt = Gadget::getPref('exttarget')) {
                $target = $exttgt;
            }
            if ($redirect = Gadget::getConf('Gadget', 'redirect')) {
                $link = $redirect .'?uri='. urlencode($link);
            }
        }
        
        return sprintf(
            '<a href="%s" class="%s" target="_%s">%s</a>', 
            $link, $class, $target, $title
        );
    }
    // }}}
    
    /**
     * Shorten Text
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $text
     * @param   int     $width
     */
    function shortenText($text, $width = 72)
    {
        if ($width < strlen($text)) {
            $thrd = floor($width / 3 - 1);
            $text = substr($text, 0, $thrd * 2) .'...'. substr($text, -$thrd);
        }
        return $text;
    }
    
    /**
     * Quote Text
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $string
     * @param   int     $width
     * @param   string  $char
     */
    function quoteText($string, $width = 72, $char = '>')
    {
        if (!strlen($char)) {
            return wordwrap($string, $width);
        }
        
        $final = '';
        $lines = array_map('rtrim', explode("\n", $string));
        $whalf = round($width-- / 2);
        
        while (null !== $line = array_shift($lines)) {
            $over = array();
            $qlen = strspn($line, '>');
            $slen = strcspn($line, ' ');
            $scnt = substr_count($line, ' ');
            
            // new line would be to long
            if (strlen($line) >= $width && $scnt > 1 && $qlen < $whalf && $slen < $whalf) {
                $parts = explode(' ', $line);
                do {
                    array_unshift($over, array_pop($parts));
                } while (strlen($line = implode(' ', $parts)) >= $width);
            }
            
            $final .= $char;
            if (isset($line{0}) && $line{0} != $char) {
                $final .= ' ';
            }
            $final .= $line ."\n";
            
            if (count($over)) {
                if (isset($lines[0]) && strspn($lines[0], '>') == $qlen) {
                    $quotes = '';
                    $text = '';
                    sscanf($lines[0], "%[> ]%[^\n]", $quotes, $text);
                    if (!$quotes) {
                        $text = $lines[0];
                    }
                    $lines[0] = $quotes . implode(' ', $over);
                    if (($text = rtrim($text)) && $text{0} != ' ') {
                        $lines[0] .= ' ';
                    }
                    $lines[0] .= $text;
                } else {
                    array_unshift($lines, substr($line, 0, $qlen) .' '. implode(' ', $over));
                }
            }
        }
        return $final;
    }

    // {{{ Several Formatters
    /**
     * Format DateTime
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string
     * @param   string
     * @param   string
     */
    function formatDateTime($datetime, $mode = 'default', $pre = ' am ', $join = ' um ')
    {
        static $dates;
        
        if (!isset($dates)) {
            $dates['today'] = date('Y-m-d');
            $dates['yesterday'] = date('Y-m-d', strtotime('yesterday'));
        }
        
        if (is_numeric($datetime)) $datetime = date('Y-m-d H:i:s', $datetime);
        list($date, $time) = explode(' ', $datetime);
        
        if ($date == $dates['today']) {
            return _('heute') . $join . substr($time, 0, -3);
        }
        if ($date == $dates['yesterday']) {
            return _('gestern') . $join . substr($time, 0, -3);
        }
        
        $unixts = strtotime($datetime);
        return $pre . Gadget_Formatter::formatDate($unixts, $mode) . $join . Gadget_Formatter::formatTime($unixts);
    }
    
    /**
     * Format Date
     * 
     * @static
     * @access  public
     * @return  string
     * @param   int
     * @param   string
     */
    function formatDate($ts, $mode = 'default')
    {
        if (!is_numeric($ts)) $ts = strtotime($ts);
        $locale = &Gadget::locale();
        $locale->setDateFormat(constant('I18Nv2_DATETIME_'. strToUpper($mode)));
        return $locale->formatDate($ts);
    }
    
    /**
     * Format Time
     * 
     * @static
     * @access  public
     * @return  string
     * @param   int
     */
    function formatTime($ts)
    {
        if (!is_numeric($ts)) $ts = strtotime($ts);
        $locale = &Gadget::locale();
        $locale->setTimeFormat(I18Nv2_DATETIME_SHORT);
        return $locale->formatTime($ts);
    }
    
    /**
     * Format Number
     * 
     * @static
     * @access  public
     * @return  string
     */
    function formatNumber($num, $fmt = null)
    {
        $locale = &Gadget::locale();
        return $locale->formatNumber($num, $fmt);
    }
    
    /**
     * Format Language
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $code
     */
    function formatLanguage($code)
    {
        return Gadget_Formatter::formatCLC('Language', $code);
    }
    
    /**
     * Format Country
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $code
     */
    function formatCountry($code)
    {
        return Gadget_Formatter::formatCLC('Country', $code);
    }
    
    /**
     * Format Currency
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $code
     */
    function formatCurrency($code)
    {
        return Gadget_Formatter::formatCLC('Currency', $code);
    }
    
    /**
     * Format Country, Language or Currency
     * 
     * @static
     * @access  protected
     * @return  string
     * @param   string  $CLC
     * @param   string  $code
     */
    function formatCLC($CLC, $code)
    {
        $lang = Gadget::getPref('language');
        $list = &Gadget::persistentObject('I18Nv2_'. ucFirst($CLC), $lang);
        $list->setEncoding(GADGET_CHARSET);
        
        if ($list->language != $lang) {
            $list->setLanguage($lang);
        }
        
        return $list->getName($code);
    }
    
    /**
     * Format Email
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $email
     * @param   string  $mailto
     */
    function formatEmail($email, $mailto = 'mailto:')
    {
        $host = Gadget::getConf('mailhost');
        if (function_exists('imap_rfc822_parse_adrlist')) {
            if (!$addresses = @imap_rfc822_parse_adrlist($email, $host)) {
                return $email;
            }
        } else {
            require_once 'Mail/RFC822.php';
            $addresses = Mail_RFC822::parseAddressList($email, $host);
            if (PEAR::isError($addresses)) {
                return $email;
            }
        }
        
        $maillinks = '';
        foreach ($addresses as $a) {
            $mail = $a->mailbox .'@'. $a->host;
            
            if (empty($a->personal)) {
                $personal   = $mail;
                $additional = '';
            } else {
                if (function_exists('imap_utf8')) {
                    $personal   = iconv('utf-8', GADGET_CHARSET."//TRANSLIT", imap_utf8($a->personal));
                    $additional = trim($personal, '"');
                } else {
                    $personal   = $a->personal;
                    $additional = trim($a->personal, '"');
                }
            }
            
            $maillinks .= sprintf('<a href="%s%s?name=%s">%s</a> ',
                $mailto, urlencode($mail), urlencode($additional), $personal);
        }
        return $maillinks;
    }
    
    /**
     * Format Bytes
     * 
     * @static
     * @access  public
     * @return  string
     * @param   int     $bytes
     */
    function formatBytes($bytes)
    {
        if ($bytes > 1099511627776) {
            $f = 1099511627776;
            $s = 'TB';
        } elseif ($bytes > 1073741824) {
            $f = 1073741824;
            $s = 'GB';
        } elseif ($bytes > 1048576) {
            $f = 1048576;
            $s = 'MB';
        } else {
            $f = 1024;
            $s = 'KB';
        }
        $locale = &Gadget::locale();
        return $locale->formatNumber($bytes / $f) .' '. $s;
    }
    // }}}
    
    // {{{ Others
    /**
     * To Percent
     * 
     * @static
     * @access  public
     * @return  string
     * @param   int     $fract
     * @param   int     $total
     */
    function toPercent($fract, $total = 100, $plain = false)
    {
        $percent = $total ? $fract * 100 / $total : 0;
        if ($plain) {
            return number_format($percent, 2, '.', '');
        } else {
            $locale = &Gadget::locale();
            return $locale->formatNumber($percent);
        }
    }
    
    /**
     * Random Background Color
     * 
     * @static
     * @access  public
     * @return  string
     */
    function randomBgColor()
    {
        static $colors;
        static $ccount;
        
        if (!$colors) {
            $rgb = array('9', 'a', 'b', 'c', 'd');
            foreach ($rgb as $r) {
                foreach ($rgb as $g) {
                    foreach ($rgb as $b) {
                        $colors[] = $r.$g.$b;
                    }
                }
            }
            array_shift($colors);
            array_pop($colors);
            shuffle($colors);
            $ccount = count($colors) - 1;
        }
        return '#'. $colors[mt_rand(0, $ccount)];
    }
    // }}}
}
?>
