<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Mail
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.13 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Message.php,v 1.13 2010-11-16 06:30:56 mike Exp $
 */

/**
 * Gadget_Mail_Message
 *
 * Mail message facility.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Mail
 * @since       1.0
 */
class Gadget_Mail_Message
{
    var $imapConnection;
    var $imapMessageId;
    
    var $isHtml = false;
    var $isPlain = false;
    var $body = array();

    var $isSigned = false;
    var $signature = array();
    
    var $attachments = array();
    
    var $date;
    var $udate;
    
    var $subject;
    var $messageId;
    var $references;
    var $inReplyTo;
    
    var $to = array();
    var $from = array();
    var $cc = array();
    var $bcc = array();
    var $sender = array();
    var $replyTo = array();
    var $returnPath = array();
    
    var $flags = array(
        'recent'    => ' ',
        'unseen'    => ' ',
        'answered'  => ' ',
        'deleted'   => ' ',
        'draft'     => ' ',
        'flagged'   => ' '
    );
    
    /**
     * Get an IMAP Message
     * 
     * @static
     * @access  public
     * @return  object      Gadget_Mail_Message
     * @param   resource    $connection
     * @param   int         $message
     */
    function &fetch($connection, $message)
    {
        $msg = &new Gadget_Mail_Message;
        
        $msg->imapConnection = $connection;
        $msg->imapMessageId  = $message;
        
        if ($headers = imap_headerinfo($connection, imap_msgno($connection, $message))) {
            $msg->flags = array(
                'recent'    => $headers->Recent,
                'unseen'    => $headers->Unseen,
                'answered'  => $headers->Answered,
                'deleted'   => $headers->Deleted,
                'draft'     => $headers->Draft,
                'flagged'   => $headers->Flagged
            );
            
            isset($headers->date)           and $msg->date          = $headers->date;
            isset($headers->udate)          and $msg->udate         = $headers->udate;
            isset($headers->subject)        and $msg->subject       = $headers->subject;
            isset($headers->message_id)     and $msg->messageId     = $headers->message_id;
            isset($headers->references)     and $msg->references    = $headers->references;
            isset($headers->to)             and $msg->to            = $headers->to;
            isset($headers->from)           and $msg->from          = $headers->from;
            isset($headers->cc)             and $msg->cc            = $headers->cc;
            isset($headers->bcc)            and $msg->bcc           = $headers->bcc;
            isset($headers->sender)         and $msg->sender        = $headers->sender;
            isset($headers->reply_to)       and $msg->replyTo       = $headers->reply_to;
            isset($headers->return_path)    and $msg->returnPath    = $headers->return_path;
            isset($headers->in_reply_to)    and $msg->inReplyTo     = $headers->in_reply_to;
        }
        
        if ($struct = imap_fetchstructure($connection, $message, FT_UID)) {
            $msg->extract($struct);
        }
        
        return $msg;
    }
    
    /**
     * Add an Address
     * 
     * Add an address to either of To, From, CC, BCC, Sender, ReplyTo and
     * ReturnPath headers.
     * 
     * @access  public
     * @return  bool
     * @param   string  $type       header name
     * @param   string  $address    address to add
     * @param   string  $defaultHost
     */
    function addAddress($type, $address, $defaultHost = 'localhost')
    {
        if (!$address = imap_rfc822_parse_adrlist($address, $defaultHost)) {
            return false;
        }
        switch ($type = strToLower($type))
        {
            case 'to':
            case 'from':
            case 'cc':
            case 'bcc':
            case 'sender':
                $this->{$type}[] = $address;
            break;
            
            case 'replyto':
            case 'reply_to':
                $this->replyTo[] = $address;
            break;
            
            case 'returnpath':
            case 'return_path':
                $this->returnPath[] = $address;
            break;
            
            default:
                return false;
        }
        
        return true;
    }
    
    /**
     * Get Addresses
     * 
     * Get a pretty printed list either of To, From, CC, BCC, Sender, ReplyTo 
     * or ReturnPath header. (Foo Bar <foo@bar.com>, bar@$defaultHost)
     * 
     * @access  public
     * @return  string
     * @param   string  $type       header name
     * @param   string  $defaultHost
     */
    function getAddress($type, $defaultHost = 'localhost')
    {
        switch ($type = strToLower($type))
        {
            case 'to':
            case 'from':
            case 'cc':
            case 'bcc':
            case 'sender':
                $addresses = $this->{$type};
            break;
            
            case 'replyto':
            case 'reply_to':
                $addresses = $this->replyTo;
            break;
            
            case 'returnpath':
            case 'return_path':
                $addresses = $this->returnPath;
            break;
            
            default:
                return false;
        }
        
        $string = '';
        
        foreach ($addresses as $address) {
            isset($address->host) or $address->host = $defaultHost;
            if (isset($address->personal) && strlen($address->personal)) {
                $string .=  $address->personal  .' <'.
                            $address->mailbox   .'@'.
                            $address->host      .'>, ';
                            
            } else {
                $string .=  $address->mailbox .'@'. $address->host .', ';
            }
        }
        
        return substr($string, 0, -2);
    }
    
    /**
     * Set Date
     * 
     * @access  public
     * @return  bool
     * @param   int|string  $date
     */
    function setDate($date)
    {
        if (0 > $time = (is_numeric($date) ? $date : strtotime($date))) {
            return false;
        }
        
        $this->date  = gmtime('r', $time);
        $this->udate = $time;
        
        return true;
    }
    
    /**
     * Get Date when Message was Sent
     * 
     * @access  public
     * @return  string
     */
    function getSentDate()
    {
        return $this->date ? $this->date : gmdate('r', 0);
    }
    
    /**
     * Get Date when Message arrived
     * 
     * @access  public
     * @return  string|int
     */
    function getArrivalDate($unix = false)
    {
        $time = $this->udate ? $this->udate : time();
        return $unix ? $time : gmdate('r', $time);
    }
    
    /**
     * Get Message Date
     * 
     * Returns the arrival date if present, or the sent date if present;
     * otherwise current date.
     * 
     * @access  public
     * @return  string
     */
    function getDate()
    {
        if (!$this->udate) {
            if (!$this->date) {
                return gmdate('r');
            }
            return $this->date;
        }
        return gmdate('r', $this->udate);
    }
    
    /**
     * Get Subject
     * 
     * Get the literal (mime decoded) subject of the message.
     * 
     * @access  public
     * @return  string
     */
    function getSubject()
    {
        return Gadget_Mail::decodeMIME($this->subject);
    }

    /**
     * Get a Message Flag
     * 
     * @access  public
     * @return  string
     */
    function getFlag($type)
    {
        $type = strToLower($type);
        return isset($this->flags[$type]) ? $this->flags[$type] : null;
    }
    
    /**
     * Is New
     * 
     * @access  public
     * @return  bool
     */
    function isNew()
    {
        return $this->flags['unseen'] == 'U' || $this->flags['recent'] == 'N';
    }
    
    /**
     * Get HTML or PlainText Body
     * 
     * @access  public
     * @return  string
     */
    function getBody()
    {
        switch (true)
        {
            case $this->isHtml:     $type = 'HTML';     break;
            case $this->isPlain:    $type = 'PLAIN';    break;
            default: return '';
        }
        if (!isset($this->body[$type]['encoded'])) {
            $this->body[$type]['encoded'] = imap_fetchbody(
                $this->imapConnection, $this->imapMessageId,
                $this->body[$type]['id'] ? $this->body[$type]['id'] : 1,
                FT_UID
            );
        }
        if (!isset($this->body[$type]['decoded'])) {
            $decoded = $this->decode(
                $this->body[$type]['encoded'],
                $this->body[$type]['encoding']
            );
            if ($this->body[$type]['charset']) {
                $decoded = iconv($this->body[$type]['charset'], 
                    GADGET_CHARSET."//TRANSLIT", $decoded);
            }
            $this->body[$type]['decoded'] = $decoded;
        }
        return $this->body[$type]['decoded'];
    }
    
    /**
     * Get PlainText Body
     * 
     * @access  public
     * @return  string
     */
    function getPlainBody()
    {
        if (!($body = $this->getBody()) || !$this->isHtml || $this->isPlain) {
            return $body;
        }
        return html_entity_decode(
            strip_tags(
                preg_replace(
                    array('/\r?\n/', '/<p>/', '/<br( \/)?>/', '/(\r?\n){3,}/', '/<a[^>]+href="?([^" >]+)"?[^>]*>/'), 
                    array(' ', "\n\n", "\n", "\n\n", '$1 '),
                    $body
                )
            ), ENT_QUOTES, GADGET_CHARSET
        );
    }
    
    /**
     * Get an Attachment by Index (not PartId)
     * 
     * @access  public
     * @return  object  Gadget_Mail_Attachment
     * @param   int     $idx
     */
    function &getAttachment($idx)
    {
        if (!isset($this->attachments[$idx])) {
            return PEAR::raiseError("No attachment at offset $idx");
        }
        if ($this->attachments[$idx]->imap) {
            if (!isset($this->attachments[$idx]->encoded)) {
                $this->attachments[$idx]->encoded = imap_fetchbody(
                    $this->imapConnection, 
                    $this->imapMessageId,
                    $this->attachments[$idx]->id,
                    FT_UID
                );
            }
            if (!isset($this->attachments[$idx]->decoded)) {
                $this->attachments[$idx]->decoded = $this->decode(
                    $this->attachments[$idx]->encoded,
                    $this->attachments[$idx]->encoding
                );
            }
        }
        return $this->attachments[$idx];
    }

    /**
     * @todo
     */
    function toString($force_new_date = false)
    {
        if ($this->imapConnection && $this->imapMessageId) {
            return  imap_fetchheader($this->imapConnection, $this->imapMessageId, FT_UID)
                .   imap_body($this->imapConnection, $this->imapMessageId, FT_UID);
        }
        
        $hdrs = array('Date' => $this->getDate());
        
        $this->subject      and $hdrs['Subject']        = $this->subject;
        $this->messageId    and $hdrs['Message-Id']     = $this->messageId;
        $this->references   and $hdrs['References']     = $this->references;
        $this->inReplyTo    and $hdrs['In-Reply-To']    = $this->inReplyTo;
        $this->to           and $hdrs['To']             = $this->getAddress('to');
        $this->from         and $hdrs['From']           = $this->getAddress('from');
        $this->cc           and $hdrs['Cc']             = $this->getAddress('cc');
        $this->bcc          and $hdrs['Bcc']            = $this->getAddress('bcc');
        $this->sender       and $hdrs['Sender']         = $this->getAddress('sender');
        $this->replyTo      and $hdrs['Reply-To']       = $this->getAddress('reply_to');
        $this->returnPath   and $hdrs['Return-Path']    = $this->getAddress('return_path');
        
        require_once 'Mail/mime.php';
        
        $mime = new Mail_mime;
        if ($this->isHTML) {
            $mime->setHTMLBody($this->getHtmlBody());
        } else {
            $mime->setTXTBody($this->getPlainBody());
        }
        
        $c = count($this->attachments);
        for ($i = 0; $i < $c; $i++) {
            $a = &$this->getAttachment($i);
            $mime->addAttachment($a->decoded, $a->type, $a->filename, false);
        }
        
        $data = $mime->get(array('text_charset' => GADGET_CHARSET, 'html_charset' => GADGET_CHARSET));
        $hdrs = $mime->headers($hdrs);
        
        return implode("\n", $hdrs) ."\n\n". $data;
    }
    
    /**
     * Flag the Message as Answered
     * 
     * @access  public
     * @return  bool
     */
    function flagAnswered()
    {
        return imap_setflag_full($this->imapConnection, imap_msgno($this->imapConnection, $this->imapMessageId), '\\Answered');
    }
    
    /**
     * Extracts IMAP Messages handling Text, Multipart & Attachments
     * 
     * @access  protected
     * @return  void
     * @param   object  $struct
     */
    function extract($struct, $id = '')
    {
        switch ($struct->type)
        {
            // text
            case 0:
                if ($this->isPlain || $this->isHtml) {
                    $this->extractAttachment($struct, $id);
                } else {
                    $this->extractText($struct, $id);
                }
            break;
            
            // multipart
            case 1:
                $this->extractMultipart($struct, $id);
            break;
            
            // attachments
            default:
                $this->extractAttachment($struct, $id);
            break;
        }
    }
    
    /**
     * Extracts an Attachment of an IMAP Multipart Message
     * 
     * @access  protected
     * @return  void
     * @param   object  $struct
     * @param   string  $id
     */
    function extractAttachment($struct, $id)
    {
        $subt = $struct->ifsubtype ? $struct->subtype : null;
        $type = $this->typeToString($struct->type, $subt);
        $size = isset($struct->size) ? $struct->size : (isset($struct->bytes) ? $struct->bytes : 0);
        
        
            require_once 'Gadget/Mail/Attachment.php';
        $attachment = &new Gadget_Mail_Attachment(
            array(
                'imap'          => true,
                'id'            => ltrim($id, '.'),
                'type'          => $type,
                'size'          => $size,
                'encoding'      => $struct->encoding,
                'description'   => $struct->ifdescription ? $struct->description : '',
            )
        );
        
        // file name
        if ($struct->ifdparameters) {
            foreach ($struct->dparameters as $p) {
                switch ($p->attribute)
                {
                    case 'filename':
                        $attachment->filename = Gadget_Mail::decodeMime($p->value);
                    break;
                }
            }
        }
        
        // part name
        if ($struct->ifparameters) {
            foreach ($struct->parameters as $p) {
                switch ($p->attribute)
                {
                    case 'name':
                        $attachment->name = Gadget_Mail::decodeMime($p->value);
                    break;
                }
            }
        }
        
        // catch pgp-signature
        if ($subt == 'PGP-SIGNATURE') {
            $this->signature['attachment'] = &$attachment;
        } else {
            $this->attachments[] = &$attachment;
        }
    }
    
    /**
     * Attempts to extract the SubParts of an IMAP Multipart Message
     * 
     * @access  protected
     * @return  void
     * @param   object  $struct
     * @param   string  $id
     */
    function extractMultipart($struct, $id)
    {
        if ($struct->subtype == 'SIGNED') {
            $this->isSigned = true;
            // extract sig info
            if ($struct->ifparameters) {
                foreach ($struct->parameters as $p) {
                    switch ($p->attribute)
                    {
                        case 'micalg':
                        case 'protocol':
                            $this->signature[$p->attribute] = $p->value;
                        break;
                    }
                }
            }
        }
        if (!empty($struct->parts)) {
            foreach ($struct->parts as $subid => $part) {
                // extract sub parts
                $this->extract($part, $id .'.'. ($subid + 1));
            }
        }
    }
    
    /**
     * Extracts the actual Message from the IMAP Structure
     * 
     * @access  protected
     * @return  void
     * @param   object  $struct
     * @param   string  $id
     */
    function extractText($struct, $id)
    {
        // get text type; don't overwrite message type, if exists
        switch ($struct->subtype)
        {
            case 'PLAIN':
                $this->isHtml or $this->isPlain = true;
            break;
            
            case 'HTML':
                $this->isPlain or $this->isHtml = true;
            break;
            
            default:
                $this->extractImapAttachment($struct, $id);
                // don't do anything more
                return;
        }
        
        // for fetchbody
        $this->body[$struct->subtype] = array(
            'id'        => ltrim($id, '.'),
            'encoding'  => $struct->encoding,
            'charset'   => null,
            'encoded'   => null,
            'decoded'   => null,
        );
        
        // get charset
        if ($struct->ifparameters) {
            foreach ($struct->parameters as $p) {
                switch ($p->attribute)
                {
                    case 'charset':
                        $this->body[$struct->subtype][$p->attribute] = $p->value;
                    break;
                }
            }
        }
    }
    
    /**
     * Decode
     * 
     * @access  protected
     * @return  string
     * @param   string      $data
     * @param   int         $encoding
     */
    function decode($data, $encoding)
    {
        switch ($encoding)
        {
            // 7bit
            case 0:
            // 8bit
            case 1:
            // binary
            case 2:
                return $data;
            
            // base64
            case 3:
                return base64_decode($data);
            
            // QP
            case 4:
                return imap_qprint($data);
                
            // unknown
            case 5:
            default:
                return $data;
        }
    }
    
    /**
     * Get the full Content Type of a Message Part
     * 
     * @access  public
     * @return  string
     * @param   int     $p
     * @param   string  $s
     */
    function typeToString($p, $s)
    {
        static $types = array('text', 'multipart', 'message', 'application',
            'audio', 'image', 'video', 'other');
        $type = isset($types[$p]) ? $types[$p] : 'unknown';
        if ($s) {
            $type .= '/'. strToLower($s);
        }
        return $type;
    }
}
?>
