<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.11 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Upload.php,v 1.11 2007-03-30 11:40:07 mike Exp $
 */

/**
 * Requires Gadget_Object
 */
require_once 'Gadget/Object.php';

/**#@+
 * Error Constants
 */
define('UPLOAD_ERROR_DEST_NOT_WRITABLE',    1);
define('UPLOAD_ERROR_DEST_ALREADY_EXISTS',  2);
define('UPLOAD_ERROR_NOT_MOVED',            3);
/**#@-*/

/**
 * Gadget_Upload
 * 
 * @package Gadget
 * @since   1.0
 */
class Gadget_Upload extends Gadget_Object
{
    /**
     * File Extension
     * 
     * @var string
     */
    var $ext;
    
    /**
     * File Name
     * 
     * @var string
     */
    var $name;
    
    /**
     * File Type
     * 
     * @var string
     */
    var $type;
    
    /**
     * Temporary Name
     * 
     * @var string
     */
    var $tmp_name;
    
    /**
     * File Size in Bytes
     * 
     * @var int
     */
    var $size;
    
    /**
     * Error Code
     * 
     * @var int
     */
    var $error;
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     */
    function onAfterConstruct()
    {
        $this->ext = strToLower(substr($this->name, strrpos($this->name, '.') + 1));
    }
    
    /**
     * Get Uploaded File
     * 
     * Usage:
     * <pre>
     * while ($file = &Gadget_Upload::getFile('images')) {
     *     if ($file->checkSize(50000)) {
     *         $file->nameToSafe();
     *         $file->moveTo('./uploads');
     *     }
     * }
     * </pre>
     * 
     * @static
     * @access  public
     * @return  mixed
     * @param   string  $name
     */
    function &getFile($name)
    {
        static $multiple;
        $upload = null;
        isset($multiple[$name]) or $multiple[$name] = 0;
        if (isset($_FILES[$name])) {
            if (is_array($_FILES[$name]['tmp_name'])) {
                if (isset($_FILES[$name]['tmp_name'][$multiple[$name]]) &&
                    strlen($_FILES[$name]['name'][$multiple[$name]])) {
                    $file = $multiple[$name]++;
                    $upload = &new Gadget_Upload(
                        array(
                            'name'      => $_FILES[$name]['name'][$file],
                            'size'      => $_FILES[$name]['size'][$file],
                            'type'      => $_FILES[$name]['type'][$file],
                            'error'     => $_FILES[$name]['error'][$file],
                            'tmp_name'  => $_FILES[$name]['tmp_name'][$file],
                        )
                    );
                }
            } elseif (!$multiple[$name]++ && $_FILES[$name]['tmp_name']) {
                $upload = &new Gadget_Upload($_FILES[$name]);
            }
        }
        return $upload;
    }
    
    /**
     * Move uploaded File
     * 
     * @access  pubic
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $destination just the directory!
     * @param   bool    $overwrite
     */
    function moveTo($destination, $overwrite = true)
    {
        if (!is_writable($destination)) {
            return PEAR::raiseError("$destiantion is not writable",
                UPLOAD_ERROR_DEST_NOT_WRITABLE);
        }
        if (!$overwrite && is_file($destination .'/'. $this->name)) {
            return PEAR::raiseError("$destination/{$this->name} already exists",
                UPLOAD_ERROR_DEST_ALREADY_EXISTS);
        }
        if (!@move_uploaded_file($this->tmp_name, $destination .'/'. $this->name)) {
            return PEAR::raiseError("Could not move {$this->name} to $destination",
                UPLOAD_ERROR_NOT_MOVED);
        }
        @chmod($destination .'/'. $this->name, 0660);
        return true;
    }
    
    /**
     * Check File Size
     * 
     * @access  public
     * @return  bool
     * @param   mixed   $size
     */
    function checkSize($size)
    {
        $size = preg_replace(array('/m/i','/k/i'), array('000000','000'), $size);
        return $this->size <= (int) $size;
    }
    
    /**
     * Check File Extension
     * 
     * @access  public
     * @return  bool
     * @param   array   $exts
     * @param   bool    $deny
     */
    function checkExt($exts, $deny = false)
    {
        return in_array($this->ext, $exts) ? !$deny : $deny;
    }
    
    /**
     * Check File Type
     * 
     * @access  public
     * @return  bool
     * @param   string  $type
     */
    function checkType($type)
    {
        if (false !== strchr($type, '/')) {
            return !strcasecmp($this->type, $type);
        } else {
            return !strcasecmp(substr($this->type, 0, strpos($this->type, '/')), $type);
        }
    }
    
    /**
     * Name to Uniq
     * 
     * @access  public
     * @return  string  new name
     */
    function nameToUniq($prefix = 'upload')
    {
        return $this->name = uniqid($prefix, true) .'.'. $this->ext;
    }
    
    /**
     * Name to Safe
     * 
     * @access  public
     * @return  string  new name
     */
    function nameToSafe()
    {
        return $this->name = preg_replace('/[^\w.-]/', '_', 
            iconv(GADGET_CHARSET, 'ASCII//IGNORE', $this->name));
    }
}

?>
